=begin rdoc

Versionize provides a simple version management and reporting interface for
Ruby projects.  Include the Versionize module in your class, set the @version
variable, and start using the version method.  Away you go.

=end

module Versionize
  def self.included(base)
    base.extend(ClassMethods)
  end

=begin

The +@version" hash is the core of Versionize.  Set the values of the three
sub-versions (+:major+, +:minor+, and +:revision+) to desired values.

=end

  @version = {
    :major      =>  1,
    :minor      =>  0,
    :revision   =>  2
  }

  module ClassMethods

=begin

The +version+ method takes one optional argument +format+, indicating the type
of data you want the method to return (+:array+, +:hash+, or +:string+).  The
default format is +:string+.  The return value includes all three sub-versions.

Examples:

        Versionize.version
        => "0.0.1"

        Versionize.version(:array)
        => [0,0,1]

        Versionize.version(:hash)
        => {:major=>0,:minor=>0,:revision=>1}

=end

    def version(format=:string)
      case format
      when :array
        [ @version[:major], @version[:minor], @version[:revision] ]
      when :hash
        @version
      when :string
        version(:array).collect {|n| n.to_s }.join '.'
      end
    end

=begin

The +major+ method takes one optional argument +format+, indicating the type of
data you want the method to return (+:integer+ or +:string+).  The default
format is +:integer+.  The return value includes only the +:major+ sub-version.

Examples:

        Versionize.major
        => 0

        Versionize.major(:string)
        => "0"

=end

    def major(format=:integer)
      case format
      when :integer
        @version[:major]
      when :string
        @version[:major].to_s
      end
    end

=begin

The +minor+ method operates just like the +major+ method except that its return
value includes only the +:minor+ sub-version.

=end

    def minor(format=:integer)
      case format
      when :integer
        @version[:minor]
      when :string
        @version[:minor].to_s
      end
    end

=begin

The +revision+ method operates just like the +major+ method except that its
return value includes only the +:revision+ sub-version.

=end

    def revision(format=:integer)
      case format
      when :integer
        @version[:revision]
      when :string
        @version[:revision].to_s
      end
    end
  end

  extend ClassMethods
end
